/**
 * Copyright (C) 2001 Billy Biggs.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef MIDICONTROL_H_INCLUDED
#define MIDICONTROL_H_INCLUDED

#include <thread.h>
#include <rtctimer.h>
#include <mididev.h>

class SongChannel;
class SongPattern;

class MidiControl : public Thread
{
public:
	MidiControl( void );
	~MidiControl( void );

	bool isRealtime( void ) const { return isrt; }

	void useInternalClock( void ) { useintclock = true; }
	void useExternalClock( void ) { useintclock = false; }

	bool beatPulse( void ) const { return bp; }

private:
	MidiDev *mididev;
	bool useintclock;
	bool isrt;
	bool bp;
	int bcount;

	// Last sec/usec we sent a sync signal.
	int last_sec;
	int last_usec;
	int last_diff;

	// RTC device.
	RealTimeClock *rtc;

	// Our cheezy MIDI tick counter (24ppq for now).
	int midiclockcounter;

	// Ugly hacked state information.
	bool wasplaying;

	// Do the next beat using RTC sync.
	void rtcTimeCheck( void );

	// Check if we're into the next beat when using external midi sync.
	void midiTimeCheck( void );

	// This function acutally plays the current noteset.
	void doCurrentNotes( void );

	// Play a note out a channel.
	void doNote( SongChannel *chan, SongPattern *pat, unsigned int beat );

	// Sent an allNotesOff message on every MIDI channel.
	void shutUp( void );

	// Thread loop.
	void thread_main( void );
};

#endif // MIDICONTROL_H_INCLUDED
